import argparse

import meshio

from ..main import remesh_surface as rms
from .helpers import _get_version_text


def remesh_surface(argv=None):
    parser = _get_parser()
    args = parser.parse_args(argv)

    mesh = rms(
        args.infile,
        # lloyd=args.lloyd,
        # odt=args.odt,
        # perturb=args.perturb,
        # exude=args.exude,
        max_edge_size_at_feature_edges=args.max_edge_size_at_feature_edges,
        min_facet_angle=args.min_facet_angle,
        max_radius_surface_delaunay_ball=args.max_radius_surface_delaunay_ball,
        max_facet_distance=args.max_facet_distance,
        verbose=not args.quiet,
    )
    meshio.write(args.outfile, mesh)


def _get_parser():
    parser = argparse.ArgumentParser(
        description="Remesh surface mesh.",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument("infile", type=str, help="input mesh file")
    parser.add_argument("outfile", type=str, help="output mesh file")

    parser.add_argument(
        "--max-edge-size-at-feature-edges",
        "-e",
        type=float,
        default=0.0,
        help="maximum edge size at feature edges (default: 0.0)",
    )

    parser.add_argument(
        "--min-facet-angle",
        "-a",
        type=float,
        default=0.0,
        help="minimum facet angle (default: 0.0)",
    )

    parser.add_argument(
        "--max-radius-surface-delaunay-ball",
        "-s",
        type=float,
        default=0.0,
        help="maximum radius of the surface facet Delaunay ball (default: 0.0)",
    )

    parser.add_argument(
        "--max-facet-distance",
        "-d",
        type=float,
        default=0.0,
        help="maximum facet distance (default: 0.0)",
    )

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        default=False,
        help="quiet mode (default: False)",
    )

    parser.add_argument(
        "--version", "-v", action="version", version=_get_version_text()
    )
    return parser



import numpy
from helpers import compute_triangle_areas

import pygalmesh


def test_rectangle():
    points = numpy.array([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0]])
    constraints = [[0, 1], [1, 2], [2, 3], [3, 0]]

    mesh = pygalmesh.generate_2d(
        points,
        constraints,
        max_edge_size=1.0e-1,
        num_lloyd_steps=10,
    )

    assert mesh.points.shape == (276, 2)
    assert mesh.get_cells_type("triangle").shape == (486, 3)

    # # show mesh
    # import matplotlib.pyplot as plt
    # pts = points[cells]
    # for pt in pts:
    #     plt.plot([pt[0][0], pt[1][0]], [pt[0][1], pt[1][1]], "-k")
    #     plt.plot([pt[1][0], pt[2][0]], [pt[1][1], pt[2][1]], "-k")
    #     plt.plot([pt[2][0], pt[0][0]], [pt[2][1], pt[0][1]], "-k")
    # # for pt in points:
    # #     plt.plot(pt[0], pt[1], "or")
    # plt.gca().set_aspect("equal")
    # plt.show()

    # mesh.points *= 100
    # mesh.write("rect.svg")


def test_disk():
    h = 0.1
    n = int(2 * numpy.pi / h)
    points = numpy.array(
        [
            [numpy.cos(alpha), numpy.sin(alpha)]
            for alpha in numpy.linspace(0.0, 2 * numpy.pi, n + 1, endpoint=False)
        ]
    )
    constraints = [[k, k + 1] for k in range(n)] + [[n, 0]]
    mesh = pygalmesh.generate_2d(
        points,
        constraints,
        max_edge_size=h,
        num_lloyd_steps=0,
    )
    areas = compute_triangle_areas(mesh.points, mesh.get_cells_type("triangle"))
    assert numpy.all(areas > 1.0e-5)


if __name__ == "__main__":
    test_disk()



import math
import os
import tempfile
from typing import Tuple

import meshio
import numpy
from _pygalmesh import (
    SizingFieldBase,
    _generate_2d,
    _generate_from_inr,
    _generate_from_inr_with_subdomain_sizing,
    _generate_from_off,
    _generate_mesh,
    _generate_periodic_mesh,
    _generate_surface_mesh,
    _remesh_surface,
)


class Wrapper(SizingFieldBase):
    def __init__(self, f):
        self.f = f
        super().__init__()

    def eval(self, x):
        return self.f(x)


def generate_mesh(
    domain,
    extra_feature_edges=None,
    bounding_sphere_radius: float = 0.0,
    lloyd: bool = False,
    odt: bool = False,
    perturb: bool = True,
    exude: bool = True,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    max_circumradius_edge_ratio: float = 0.0,
    max_cell_circumradius: float = 0.0,
    verbose: bool = True,
    seed: int = 0,
):
    """
    From <https://doc.cgal.org/latest/Mesh_3/classCGAL_1_1Mesh__criteria__3.html>:

    max_edge_size_at_feature_edges:
        a scalar field (resp. a constant) providing a space varying (resp. a uniform)
        upper bound for the lengths of curve edges. This parameter has to be set to a
        positive value when 1-dimensional features protection is used.
    min_facet_angle:
        a lower bound for the angles (in degrees) of the surface mesh facets.
    max_radius_surface_delaunay_ball:
        a scalar field (resp. a constant) describing a space varying (resp. a uniform)
        upper-bound or for the radii of the surface Delaunay balls.
    max_facet_distance:
        a scalar field (resp. a constant) describing a space varying (resp. a uniform)
        upper bound for the distance between the facet circumcenter and the center of
        its surface Delaunay ball.
    max_circumradius_edge_ratio:
        an upper bound for the radius-edge ratio of the mesh tetrahedra.
    max_cell_circumradius:
        a scalar field (resp. a constant) describing a space varying (resp. a uniform)
        upper-bound for the circumradii of the mesh tetrahedra.
    """
    extra_feature_edges = [] if extra_feature_edges is None else extra_feature_edges

    fh, outfile = tempfile.mkstemp(suffix=".mesh")
    os.close(fh)

    def _select(obj):
        if isinstance(obj, float):
            return obj, None
        assert callable(obj)
        return -1.0, Wrapper(obj)

    (
        max_edge_size_at_feature_edges_value,
        max_edge_size_at_feature_edges_field,
    ) = _select(max_edge_size_at_feature_edges)
    max_cell_circumradius_value, max_cell_circumradius_field = _select(
        max_cell_circumradius
    )
    (
        max_radius_surface_delaunay_ball_value,
        max_radius_surface_delaunay_ball_field,
    ) = _select(max_radius_surface_delaunay_ball)
    max_facet_distance_value, max_facet_distance_field = _select(max_facet_distance)

    # if feature_edges:
    #     if max_edge_size_at_feature_edges == 0.0:
    #         raise ValueError(
    #             "Need a positive max_edge_size_at_feature_edges bound if feature_edges are present."
    #         )
    # elif max_edge_size_at_feature_edges != 0.0:
    #     warnings.warn(
    #         "No feature edges. The max_edge_size_at_feature_edges argument has no effect."
    #     )

    _generate_mesh(
        domain,
        outfile,
        extra_feature_edges=extra_feature_edges,
        bounding_sphere_radius=bounding_sphere_radius,
        lloyd=lloyd,
        odt=odt,
        perturb=perturb,
        exude=exude,
        max_edge_size_at_feature_edges_value=max_edge_size_at_feature_edges_value,
        max_edge_size_at_feature_edges_field=max_edge_size_at_feature_edges_field,
        min_facet_angle=min_facet_angle,
        max_radius_surface_delaunay_ball_value=max_radius_surface_delaunay_ball_value,
        max_radius_surface_delaunay_ball_field=max_radius_surface_delaunay_ball_field,
        max_facet_distance_value=max_facet_distance_value,
        max_facet_distance_field=max_facet_distance_field,
        max_circumradius_edge_ratio=max_circumradius_edge_ratio,
        max_cell_circumradius_value=max_cell_circumradius_value,
        max_cell_circumradius_field=max_cell_circumradius_field,
        verbose=verbose,
        seed=seed,
    )

    mesh = meshio.read(outfile)
    os.remove(outfile)
    return mesh


def generate_2d(
    points,
    constraints,
    B: float = math.sqrt(2),
    max_edge_size: float = 0.0,
    num_lloyd_steps: int = 0,
):
    # some sanity checks
    points = numpy.asarray(points)
    constraints = numpy.asarray(constraints)
    assert numpy.all(constraints >= 0)
    assert numpy.all(constraints < len(points))
    # make sure there are no edges of 0 length
    edges = points[constraints[:, 0]] - points[constraints[:, 1]]
    length2 = numpy.einsum("ij,ij->i", edges, edges)
    if numpy.any(length2 < 1.0e-15):
        raise RuntimeError("Constraint of (near)-zero length.")

    points, cells = _generate_2d(
        points,
        constraints,
        B,
        max_edge_size,
        num_lloyd_steps,
    )
    return meshio.Mesh(numpy.array(points), {"triangle": numpy.array(cells)})


def generate_periodic_mesh(
    domain,
    bounding_cuboid,
    lloyd: bool = False,
    odt: bool = False,
    perturb: bool = True,
    exude: bool = True,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    max_circumradius_edge_ratio: float = 0.0,
    max_cell_circumradius: float = 0.0,
    number_of_copies_in_output: int = 1,
    verbose: bool = True,
    seed: int = 0,
):
    fh, outfile = tempfile.mkstemp(suffix=".mesh")
    os.close(fh)

    assert number_of_copies_in_output in [1, 2, 4, 8]

    _generate_periodic_mesh(
        domain,
        outfile,
        bounding_cuboid,
        lloyd=lloyd,
        odt=odt,
        perturb=perturb,
        exude=exude,
        max_edge_size_at_feature_edges=max_edge_size_at_feature_edges,
        min_facet_angle=min_facet_angle,
        max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
        max_facet_distance=max_facet_distance,
        max_circumradius_edge_ratio=max_circumradius_edge_ratio,
        max_cell_circumradius=max_cell_circumradius,
        number_of_copies_in_output=number_of_copies_in_output,
        verbose=verbose,
        seed=seed,
    )

    mesh = meshio.read(outfile)
    os.remove(outfile)
    return mesh


def generate_surface_mesh(
    domain,
    bounding_sphere_radius: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    verbose: bool = True,
    seed: int = 0,
):
    fh, outfile = tempfile.mkstemp(suffix=".off")
    os.close(fh)

    _generate_surface_mesh(
        domain,
        outfile,
        bounding_sphere_radius=bounding_sphere_radius,
        min_facet_angle=min_facet_angle,
        max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
        max_facet_distance=max_facet_distance,
        verbose=verbose,
        seed=seed,
    )

    mesh = meshio.read(outfile)
    os.remove(outfile)
    return mesh


def generate_volume_mesh_from_surface_mesh(
    filename: str,
    lloyd: bool = False,
    odt: bool = False,
    perturb: bool = True,
    exude: bool = True,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    max_circumradius_edge_ratio: float = 0.0,
    max_cell_circumradius: float = 0.0,
    verbose: bool = True,
    reorient: bool = False,
    seed: int = 0,
):
    mesh = meshio.read(filename)

    fh, off_file = tempfile.mkstemp(suffix=".off")
    os.close(fh)
    meshio.write(off_file, mesh)

    fh, outfile = tempfile.mkstemp(suffix=".mesh")
    os.close(fh)

    _generate_from_off(
        off_file,
        outfile,
        lloyd=lloyd,
        odt=odt,
        perturb=perturb,
        exude=exude,
        max_edge_size_at_feature_edges=max_edge_size_at_feature_edges,
        min_facet_angle=min_facet_angle,
        max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
        max_facet_distance=max_facet_distance,
        max_circumradius_edge_ratio=max_circumradius_edge_ratio,
        max_cell_circumradius=max_cell_circumradius,
        verbose=verbose,
        reorient=reorient,
        seed=seed,
    )

    mesh = meshio.read(outfile)
    os.remove(off_file)
    os.remove(outfile)
    return mesh


def generate_from_inr(
    inr_filename: str,
    lloyd: bool = False,
    odt: bool = False,
    perturb: bool = True,
    exude: bool = True,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    max_circumradius_edge_ratio: float = 0.0,
    max_cell_circumradius: float = 0.0,
    verbose: bool = True,
    seed: int = 0,
):
    fh, outfile = tempfile.mkstemp(suffix=".mesh")
    os.close(fh)

    if isinstance(max_cell_circumradius, float):
        _generate_from_inr(
            inr_filename,
            outfile,
            lloyd=lloyd,
            odt=odt,
            perturb=perturb,
            exude=exude,
            max_edge_size_at_feature_edges=max_edge_size_at_feature_edges,
            min_facet_angle=min_facet_angle,
            max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
            max_facet_distance=max_facet_distance,
            max_circumradius_edge_ratio=max_circumradius_edge_ratio,
            max_cell_circumradius=max_cell_circumradius,
            verbose=verbose,
            seed=seed,
        )
    else:
        assert isinstance(max_cell_circumradius, dict)
        if "default" in max_cell_circumradius.keys():
            default_max_cell_circumradius = max_cell_circumradius.pop("default")
        else:
            default_max_cell_circumradius = 0.0

        max_cell_circumradiuss = list(max_cell_circumradius.values())
        subdomain_labels = list(max_cell_circumradius.keys())

        _generate_from_inr_with_subdomain_sizing(
            inr_filename,
            outfile,
            default_max_cell_circumradius,
            max_cell_circumradiuss,
            subdomain_labels,
            lloyd=lloyd,
            odt=odt,
            perturb=perturb,
            exude=exude,
            max_edge_size_at_feature_edges=max_edge_size_at_feature_edges,
            min_facet_angle=min_facet_angle,
            max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
            max_facet_distance=max_facet_distance,
            max_circumradius_edge_ratio=max_circumradius_edge_ratio,
            verbose=verbose,
            seed=seed,
        )

    mesh = meshio.read(outfile)
    os.remove(outfile)
    return mesh


def remesh_surface(
    filename: str,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_facet_distance: float = 0.0,
    verbose: bool = True,
    seed: int = 0,
):
    mesh = meshio.read(filename)

    fh, off_file = tempfile.mkstemp(suffix=".off")
    os.close(fh)
    meshio.write(off_file, mesh)

    fh, outfile = tempfile.mkstemp(suffix=".off")
    os.close(fh)

    _remesh_surface(
        off_file,
        outfile,
        max_edge_size_at_feature_edges=max_edge_size_at_feature_edges,
        min_facet_angle=min_facet_angle,
        max_radius_surface_delaunay_ball=max_radius_surface_delaunay_ball,
        max_facet_distance=max_facet_distance,
        verbose=verbose,
        seed=seed,
    )

    mesh = meshio.read(outfile)
    os.remove(off_file)
    os.remove(outfile)
    return mesh


def save_inr(vol, voxel_size: Tuple[float, float, float], fname: str):
    """
    Save a volume (described as a numpy array) to INR format.
    Code inspired by iso2mesh (http://iso2mesh.sf.net) by Q. Fang
    INPUTS:
    - vol: volume as a 3D numpy array
    - h: voxel sizes
    - fname: filename for saving the inr file
    """
    fid = open(fname, "wb")

    btype, bitlen = {
        "uint8": ("unsigned fixed", 8),
        "uint16": ("unsigned fixed", 16),
        "float32": ("float", 32),
        "float64": ("float", 64),
    }[vol.dtype.name]

    xdim, ydim, zdim = vol.shape
    header = "\n".join(
        [
            "#INRIMAGE-4#{",
            f"XDIM={xdim}",
            f"YDIM={ydim}",
            f"ZDIM={zdim}",
            "VDIM=1",
            f"TYPE={btype}",
            f"PIXSIZE={bitlen} bits",
            "CPU=decm",
            f"VX={voxel_size[0]:f}",
            f"VY={voxel_size[1]:f}",
            f"VZ={voxel_size[2]:f}",
        ]
    )
    header += "\n"

    header = header + "\n" * (256 - 4 - len(header)) + "##}\n"

    fid.write(header.encode("ascii"))
    fid.write(vol.tobytes(order="F"))


def generate_from_array(
    vol,
    voxel_size: Tuple[float, float, float],
    lloyd: bool = False,
    odt: bool = False,
    perturb: bool = True,
    exude: bool = True,
    max_edge_size_at_feature_edges: float = 0.0,
    min_facet_angle: float = 0.0,
    max_radius_surface_delaunay_ball: float = 0.0,
    max_cell_circumradius: float = 0.0,
    max_facet_distance: float = 0.0,
    max_circumradius_edge_ratio: float = 0.0,
    verbose: bool = True,
    seed: int = 0,
):
    assert vol.dtype in ["uint8", "uint16"]
    fh, inr_filename = tempfile.mkstemp(suffix=".inr")
    os.close(fh)
    save_inr(vol, voxel_size, inr_filename)
    mesh = generate_from_inr(
        inr_filename,
        lloyd,
        odt,
        perturb,
        exude,
        max_edge_size_at_feature_edges,
        min_facet_angle,
        max_radius_surface_delaunay_ball,
        max_facet_distance,
        max_circumradius_edge_ratio,
        max_cell_circumradius,
        verbose,
        seed,
    )
    os.remove(inr_filename)
    return mesh



import pathlib
import tempfile

import helpers
import meshio

import pygalmesh


def test_volume_from_surface():
    this_dir = pathlib.Path(__file__).resolve().parent
    # mesh = pygalmesh.generate_volume_mesh_from_surface_mesh(
    #     this_dir / "meshes" / "elephant.vtu",
    #     min_facet_angle=25.0,
    #     max_radius_surface_delaunay_ball=0.15,
    #     max_facet_distance=0.008,
    #     max_circumradius_edge_ratio=3.0,
    #     verbose=False,
    # )
    with tempfile.TemporaryDirectory() as tmp:
        out_filename = str(pathlib.Path(tmp) / "out.vtk")
        pygalmesh._cli.volume_from_surface(
            [
                str(this_dir / "meshes" / "elephant.vtu"),
                out_filename,
                "--min-facet-angle",
                "0.5",
                "--max-radius-surface-delaunay-ball",
                "0.15",
                "--max-facet-distance",
                "0.008",
                "--max-circumradius-edge-ratio",
                "3.0",
                "--quiet",
            ]
        )
        mesh = meshio.read(out_filename)

    tol = 2.0e-2
    assert abs(max(mesh.points[:, 0]) - 0.357612477657) < tol
    assert abs(min(mesh.points[:, 0]) + 0.358747130015) < tol
    assert abs(max(mesh.points[:, 1]) - 0.496137874959) < tol
    assert abs(min(mesh.points[:, 1]) + 0.495301051456) < tol
    assert abs(max(mesh.points[:, 2]) - 0.298780230629) < tol
    assert abs(min(mesh.points[:, 2]) + 0.300472866512) < tol

    vol = sum(helpers.compute_volumes(mesh.points, mesh.get_cells_type("tetra")))
    assert abs(vol - 0.044164693065) < tol



import pathlib
import tempfile

import helpers
import meshio

import pygalmesh


def test_remesh_surface():
    this_dir = pathlib.Path(__file__).resolve().parent
    # mesh = pygalmesh.remesh_surface(
    #     this_dir / "meshes" / "lion-head.off",
    #     max_edge_size_at_feature_edges=0.025,
    #     min_facet_angle=25,
    #     max_radius_surface_delaunay_ball=0.1,
    #     max_facet_distance=0.001,
    #     verbose=False,
    # )
    with tempfile.TemporaryDirectory() as tmp:
        out_filename = str(pathlib.Path(tmp) / "out.vtk")
        pygalmesh._cli.remesh_surface(
            [
                str(this_dir / "meshes" / "lion-head.off"),
                out_filename,
                "--max-edge-size-at-feature-edges",
                "0.025",
                "--min-facet-angle",
                "25",
                "--max-radius-surface-delaunay-ball",
                "0.1",
                "--max-facet-distance",
                "0.001",
                "--quiet",
            ]
        )
        mesh = meshio.read(out_filename)

    tol = 1.0e-3
    ref = [
        3.705640e-01,
        -3.711630e-01,
        4.754940e-01,
        -4.728260e-01,
        4.998690e-01,
        -4.998350e-01,
    ]
    assert abs(max(mesh.points[:, 0]) - ref[0]) < tol * abs(ref[0])
    assert abs(min(mesh.points[:, 0]) - ref[1]) < tol * abs(ref[1])
    assert abs(max(mesh.points[:, 1]) - ref[2]) < tol * abs(ref[2])
    assert abs(min(mesh.points[:, 1]) - ref[3]) < tol * abs(ref[3])
    assert abs(max(mesh.points[:, 2]) - ref[4]) < tol * abs(ref[4])
    assert abs(min(mesh.points[:, 2]) - ref[5]) < tol * abs(ref[5])

    print(helpers.compute_triangle_areas(mesh.points, mesh.get_cells_type("triangle")))
    vol = sum(
        helpers.compute_triangle_areas(mesh.points, mesh.get_cells_type("triangle"))
    )
    ref = 1.917942005786831
    assert abs(vol - ref) < ref * 1.0e-3



import os

from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import setup

# https://github.com/pybind/python_example/
ext_modules = [
    Pybind11Extension(
        "_pygalmesh",
        # Sort input source files to ensure bit-for-bit reproducible builds
        # (https://github.com/pybind/python_example/pull/53)
        sorted(
            [
                "src/generate.cpp",
                "src/generate_2d.cpp",
                "src/generate_from_inr.cpp",
                "src/generate_from_off.cpp",
                "src/generate_periodic.cpp",
                "src/generate_surface_mesh.cpp",
                "src/remesh_surface.cpp",
                "src/pybind11.cpp",
            ]
        ),
        include_dirs=[os.environ.get("EIGEN_INCLUDE_DIR", "/usr/include/eigen3/")],
        # no CGAL libraries necessary from CGAL 5.0 onwards
        libraries=["gmp", "mpfr"],
    )
]

if __name__ == "__main__":
    setup(
        cmdclass={"build_ext": build_ext},
        ext_modules=ext_modules,
        zip_safe=False,
    )



import sys

from _pygalmesh import _CGAL_VERSION_STR

from ..__about__ import __version__


def _get_version_text():
    return "\n".join(
        [
            "pygalmesh {} [Python {}.{}.{}, CGAL {}]".format(
                __version__,
                sys.version_info.major,
                sys.version_info.minor,
                sys.version_info.micro,
                _CGAL_VERSION_STR,
            ),
            "Copyright (c) 2016-2020, Nico Schlömer <nico.schloemer@gmail.com>",
        ]
    )



import argparse

import meshio

from ..main import generate_volume_mesh_from_surface_mesh
from .helpers import _get_version_text


def volume_from_surface(argv=None):
    parser = _get_volume_from_surface_parser()
    args = parser.parse_args(argv)

    mesh = generate_volume_mesh_from_surface_mesh(
        args.infile,
        lloyd=args.lloyd,
        odt=args.odt,
        perturb=args.perturb,
        exude=args.exude,
        max_edge_size_at_feature_edges=args.max_edge_size_at_feature_edges,
        min_facet_angle=args.min_facet_angle,
        max_radius_surface_delaunay_ball=args.max_radius_surface_delaunay_ball,
        max_facet_distance=args.max_facet_distance,
        max_circumradius_edge_ratio=args.max_circumradius_edge_ratio,
        max_cell_circumradius=args.max_cell_circumradius,
        reorient=args.reorient,
        verbose=not args.quiet,
    )
    meshio.write(args.outfile, mesh)


def _get_volume_from_surface_parser():
    parser = argparse.ArgumentParser(
        description="Generate volume mesh from surface mesh.",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument("infile", type=str, help="input mesh file")

    parser.add_argument("outfile", type=str, help="output mesh file")

    parser.add_argument(
        "--lloyd",
        "-l",
        type=bool,
        default=False,
        help="Lloyd smoothing (default: false)",
    )

    parser.add_argument(
        "--odt",
        "-o",
        action="store_true",
        default=False,
        help="ODT smoothing (default: false)",
    )

    parser.add_argument(
        "--perturb",
        "-p",
        action="store_true",
        default=False,
        help="perturb (default: false)",
    )

    parser.add_argument(
        "--exude",
        "-x",
        action="store_true",
        default=False,
        help="exude (default: false)",
    )

    parser.add_argument(
        "--max-edge-size-at-feature-edges",
        "-e",
        type=float,
        default=0.0,
        help="maximum edge size at feature edges (default: 0.0)",
    )

    parser.add_argument(
        "--min-facet-angle",
        "-a",
        type=float,
        default=0.0,
        help="minimum facet angle (default: 0.0)",
    )

    parser.add_argument(
        "--max-radius-surface-delaunay-ball",
        "-s",
        type=float,
        default=0.0,
        help="maximum radius of the surface facet Delaunay ball (default: 0.0)",
    )

    parser.add_argument(
        "--max-facet-distance",
        "-d",
        type=float,
        default=0.0,
        help="maximum facet distance (default: 0.0)",
    )

    parser.add_argument(
        "--max-circumradius-edge-ratio",
        "-r",
        type=float,
        default=0.0,
        help="cell radius/edge ratio (default: 0.0)",
    )

    parser.add_argument(
        "--max-cell-circumradius",
        "-c",
        type=float,
        default=0.0,
        help="maximum cell circumradius (default: 0.0)",
    )

    parser.add_argument(
        "--reorient",
        "-t",
        action="store_true",
        default=False,
        help="automatically fix face orientation (default: False)",
    )

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        default=False,
        help="quiet mode (default: False)",
    )

    parser.add_argument(
        "--version", "-v", action="version", version=_get_version_text()
    )
    return parser
